@@ -5,6 +5,8 @@ import os
 import sys
 import re
 import math
+import json
+import uuid
 
 from pyredactkit.identifiers import Identifier
 
@@ -79,38 +81,59 @@ class Redactor:
             return False
         return mimetypes.guess_type(file)[0] in self.get_allowed_files()
 
-    def redact(self, line=str, option=str):
-        """Main function to redact
+    def write_hashmap(self, hash_map=dict, filename=str):
+        """Function that writes a .hashshadow_file.txt.json to os directory.
+        Args:
+            hash_map (dictionary): dictionary object to be written to file.
+            filename (str): name of supplied file
+
+        Returns:
+            Writes .hashshadow_file.txt.json to os directory
+        """
+        with open(f".hashshadow_{os.path.basename(filename)}.json", "w", encoding="utf-8") as file:
+            json.dump(hash_map, file)
+            print(
+                f"[ + ].hashshadow_{os.path.basename(filename)}.json file generated. Keep this safe if you need to undo the redaction.")
+
+    def valid_options(self):
+        """Function to read in valid options from Identifier.regexes
+        Args:
+            None
+
+        Returns:
+            option_tupe (tuple): redacted line
+        """
+        option_tuple = ()
+        for id in id_object.regexes:
+            option_tuple += id['type']
+        return option_tuple
+
+    def redact_specific(self, line=str, option=str, filename=str):
+        """Function to redact specific option
         Args:
             line (str) : line to be supplied to redact
             option (str): (optional) choice for redaction
+            filename (str): name of supplied file
 
         Returns:
-            redacted_line (str): redacted line
+            line (str): redacted line
         """
-        # Refactor this to loop through and check for option.
-        redacted_line = ''
-        if option in ("dns", "domain"):
-            dns = id_object.regexes[1]['pattern']
-            redacted_line = re.sub(dns, self.block, line, flags=re.IGNORECASE)
-        elif option in ("email", "emails"):
-            email = id_object.regexes[0]['pattern']
-            redacted_line = re.sub(
-                email, self.block, line, flags=re.IGNORECASE)
-        elif option in ("ip", "ipv4"):
-            ipv4 = id_object.regexes[2]['pattern']
-            redacted_line = re.sub(ipv4, self.block, line, flags=re.IGNORECASE)
-        elif option in ("cc", "creditcard"):
-            cc = id_object.regexes[3]['pattern']
-            redacted_line = re.sub(cc, self.block, line, flags=re.IGNORECASE)
-        elif option in ("nric", "fin"):
-            nric = id_object.regexes[4]['pattern']
-            redacted_line = re.sub(nric, self.block, line, flags=re.IGNORECASE)
-        elif option == "ipv6":
-            ipv6 = id_object.regexes[5]['pattern']
-            redacted_line = re.sub(ipv6, self.block, line, flags=re.IGNORECASE)
+        hash_map = {}
 
-        return redacted_line
+        for id in id_object.regexes:
+            redact_pattern = id['pattern']
+            if option in id['type'] and re.search(
+                    redact_pattern, line, flags=re.IGNORECASE):
+                pattern_string = re.search(
+                    redact_pattern, line, flags=re.IGNORECASE)
+                pattern_string = pattern_string.group(0)
+                masked_data = str(uuid.uuid4())
+                hash_map.update({masked_data: pattern_string})
+                line = re.sub(
+                    redact_pattern, masked_data, line, flags=re.IGNORECASE)
+
+        self.write_hashmap(hash_map, filename)
+        return line
 
     def redact_name(self, data=str):
         """Main function to redact
@@ -137,6 +160,8 @@ class Redactor:
             Creates redacted file.
         """
         count = 0
+        hash_map = {}
+        options_list = self.valid_options()
         try:
             # Open a file read pointer as target_file
             with open(filename, encoding="utf-8") as target_file:
@@ -170,23 +195,39 @@ class Redactor:
                             f"[ + ] No option supplied, will be redacting all the sensitive data supported")
                         for line in target_file:
                             for p in id_object.regexes:
-                                if re.search(p['pattern'], line, re.IGNORECASE):
-                                    line = re.sub(p['pattern'], self.block, line,
+                                redact_pattern = p['pattern']
+                                if re.search(redact_pattern, line, flags=re.IGNORECASE):
+                                    count += 1
+                                    pattern_string = re.search(
+                                        redact_pattern, line, flags=re.IGNORECASE)
+                                    pattern_string = pattern_string.group(0)
+                                    masked_data = str(uuid.uuid4())
+                                    hash_map.update(
+                                        {masked_data: pattern_string})
+                                    line = re.sub(redact_pattern, masked_data, line,
                                                   flags=re.IGNORECASE)
                             result.write(line)
-                            count += 1
+                        self.write_hashmap(hash_map, filename)
                     # Separate option to redact names
                     elif option in ("name", "names"):
                         content = target_file.read()
                         data = self.redact_name(content)
                         result.write(data[0])
                         count = data[1]
+                    elif option not in options_list:
+                        os.remove(
+                            f"{savedir}redacted_{os.path.basename(filename)}")
+                        sys.exit(
+                            "[ - ] Not a valid option for redaction type.")
+                    # Redacts all other options here
                     else:
                         print(f"[ + ] Redacting {option} from the file")
                         for line in target_file:
-                            line = self.redact(line, option)
+                            for id in id_object.regexes:
+                                if option in id['type'] and re.search(id['pattern'], line, flags=re.IGNORECASE):
+                                    count += 1
+                            line = self.redact_specific(line, option, filename)
                             result.write(line)
-                            count += 1
 
                     print(f"[ + ] Redacted {count} targets...")
                     print(
